@@ -26,6 +26,7 @@ from __future__ import absolute_import, print_function
 
 import apt
 import apt_pkg
+import copy
 from hashlib import md5
 import re
 import os
@@ -696,14 +697,15 @@ class SoftwareProperties(object):
     line = "deb %s %s %s" % ( repo, self.distro.codename, areas )
     return line
 
-  def add_source_from_line(self, line):
+  def add_source_from_line(self, line, enable_source_code=False):
     """
     Add a source with the given apt line and auto-add
     signing key if we have it in the whitelist
     """
     (deb_line, file) = expand_ppa_line(line.strip(), self.distro.codename)
     deb_line = self.expand_http_line(deb_line)
-    debsrc_line = 'deb-src' + deb_line[3:]
+    debsrc_entry_type = 'deb-src' if enable_source_code else '# deb-src'
+    debsrc_line = debsrc_entry_type + deb_line[3:]
     new_deb_entry = SourceEntry(deb_line, file)
     new_debsrc_entry = SourceEntry(debsrc_line, file)
     if new_deb_entry.invalid or new_debsrc_entry.invalid:
@@ -716,7 +718,7 @@ class SoftwareProperties(object):
                          comment=new_deb_entry.comment,
                          file=new_deb_entry.file,
                          architectures=new_deb_entry.architectures)
-    self.sourceslist.add(new_debsrc_entry.type,
+    self.sourceslist.add(debsrc_entry_type,
                          new_debsrc_entry.uri,
                          new_debsrc_entry.dist,
                          new_debsrc_entry.comps,
@@ -729,18 +731,44 @@ class SoftwareProperties(object):
         worker.join(30)
     return True
 
-  def remove_source(self, source):
+  def remove_source(self, source, remove_source_code=True):
     """Remove the given source"""
+    if remove_source_code:
+      if isinstance(source, str):
+        # recall this method giving a SourceEntry as an argument
+        source = self._find_source_from_string(source)
+        self.remove_source(source, True)
+      elif source is not None:
+        self.remove_source(source, False)
+        # remove the deb-src lines (enabled or not) associated to
+        # this source entry
+        source = copy.copy(source)
+        source.type = 'deb-src'
+        source.disabled = True
+        self.remove_source(source, False)
+        source.disabled = False
+        self.remove_source(source, False)
+      return
+
     # first find the source object if we got a string
     if isinstance(source, str):
       source = self._find_source_from_string(source)
+    if source is None:
+      return
     # if its a sources.list.d file and it contains only a single line
     # (the line that we just remove), then all references to that
     # file are gone and the file is not saved. we work around that
     # here
     if source.file != apt_pkg.config.find_file("Dir::Etc::sourcelist"):
       self.sourceslist.list.append(SourceEntry("", file=source.file))
-    self.sourceslist.remove(source)
+    try:
+      self.sourceslist.remove(source)
+    except ValueError:
+      # this exception is raised if trying to remove an entry that does
+      # not exist. in this case we suppress the error because there's no
+      # need to propagate it (the aim of this method is to ensure that
+      # the given entries are not listed in the sourceslist)
+      pass
     self.set_modified_sourceslist()
 
   def add_key(self, path):
