@@ -152,7 +152,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.window_main.show()
 
         # used to store the handlers of callbacks
-        self.handlers = []
+        self.handlers = {}
 
         self.apt_cache = apt.Cache()
         self.apt_client = client.AptClient()
@@ -204,8 +204,9 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         " setup the widgets that allow configuring the release upgrades "
         i = self.get_release_upgrades_policy()
         self.combobox_release_upgrades.set_active(i)
-        self.combobox_release_upgrades.connect(
-            'changed', self.on_combobox_release_upgrades_changed)
+        self.handlers[self.combobox_release_upgrades] = \
+             self.combobox_release_upgrades.connect("changed", 
+                                                    self.on_combobox_release_upgrades_changed)
 
     def init_auto_update(self):
         """ Set up the widgets that allow to configure the update automation """
@@ -236,14 +237,37 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                 self.combobox_update_interval.set_active(key)
                 break
 
-        self.handlers.append(
-            (self.combobox_update_interval,
+        self.handlers[self.combobox_update_interval] = \
              self.combobox_update_interval.connect("changed",
-                                                   self.on_combobox_update_interval_changed)))
+                                                   self.on_combobox_update_interval_changed)
 
     def show_auto_update_level(self):
         """Represent the level of update automation in the user interface"""
 
+        # Security Updates
+        self.set_security_update_level()
+
+        # Other Updates
+        if self.settings:
+            level_other = self.settings.get_int("regular-auto-launch-interval")
+            model = self.combobox_other_updates.get_model()
+            for (i, row) in enumerate(model):
+                level = model.get_value(row.iter, 1)
+                if level_other == level:
+                    self.combobox_other_updates.set_active(i)
+                    break
+
+        self.handlers[self.combobox_security_updates] = \
+             self.combobox_security_updates.connect("changed",
+                                                    self.set_sec_update_automation_level)
+
+        self.handlers[self.combobox_other_updates] = \
+             self.combobox_other_updates.connect("changed",
+                                                 self.set_other_update_automation_level)
+
+    def set_security_update_level(self):
+        """Fetch the security level, Enable/Disable and set the value appropriately"""
+        
         # Security Updates
         level_sec = self.get_update_automation_level()
         if level_sec == None:
@@ -258,26 +282,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             self.combobox_security_updates.set_active(1)  # Download automatically
         elif level_sec == softwareproperties.UPDATE_INST_SEC:
             self.combobox_security_updates.set_active(2)  # Download and install automatically
-
-        # Other Updates
-        if self.settings:
-            level_other = self.settings.get_int("regular-auto-launch-interval")
-            model = self.combobox_other_updates.get_model()
-            for (i, row) in enumerate(model):
-                level = model.get_value(row.iter, 1)
-                if level_other == level:
-                    self.combobox_other_updates.set_active(i)
-                    break
-
-        self.handlers.append(
-            (self.combobox_security_updates,
-             self.combobox_security_updates.connect("changed",
-                                                    self.set_sec_update_automation_level)))
-
-        self.handlers.append(
-            (self.combobox_other_updates,
-             self.combobox_other_updates.connect("changed",
-                                                 self.set_other_update_automation_level)))
+            
 
     def init_distro(self):
         """Setup the user interface elements to represent the distro"""
@@ -285,9 +290,9 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         # TRANS: %s stands for the distribution name e.g. Debian or Ubuntu
         self.label_dist_name.set_label(_("%s Software") % self.distro.id)
 
-        self.handlers.append((self.checkbutton_source_code,
+        self.handlers[self.checkbutton_source_code] = \
                               self.checkbutton_source_code.connect("toggled",
-                                                                   self.on_checkbutton_source_code_toggled)))
+                                                                   self.on_checkbutton_source_code_toggled)
 
         # Setup the checkbuttons for the components
         for checkbutton in self.vbox_dist_comps.get_children():
@@ -301,10 +306,9 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
 
             checkbox.comp = comp
             # setup the callback and show the checkbutton
-            self.handlers.append((checkbox,
-                                  checkbox.connect("toggled",
+            self.handlers[checkbox] = checkbox.connect("toggled",
                                                    self.on_component_toggled,
-                                                   comp.name)))
+                                                   comp.name)
             self.vbox_dist_comps.add(checkbox)
             checkbox.show()
 
@@ -321,10 +325,9 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             checkbox = Gtk.CheckButton(label="%s (%s)" % (template.description,
                                                           template.name))
             checkbox.template = template
-            self.handlers.append((checkbox,
-                                  checkbox.connect("toggled",
+            self.handlers[checkbox] = checkbox.connect("toggled",
                                                    self.on_checkbutton_child_toggled,
-                                                   template)))
+                                                   template)
             self.vbox_updates.add(checkbox)
             checkbox.show()
 
@@ -332,9 +335,8 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         cell = Gtk.CellRendererText()
         self.combobox_server.pack_start(cell, True)
         self.combobox_server.add_attribute(cell, 'text', 0)
-        self.handlers.append((self.combobox_server,
-                              self.combobox_server.connect("changed",
-                                                           self.on_combobox_server_changed)))
+        self.handlers[self.combobox_server] = self.combobox_server.connect("changed",
+                                                           self.on_combobox_server_changed)
         server_store = Gtk.ListStore(GObject.TYPE_STRING,
                                      GObject.TYPE_STRING,
                                      GObject.TYPE_BOOLEAN)
@@ -342,14 +344,14 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.combobox_server.set_row_separator_func(self.is_row_separator, 2)
 
     def block_handlers(self):
-        for (widget, handler) in self.handlers:
-            if widget.handler_is_connected(handler):
-                widget.handler_block(handler)
+        for widget in self.handlers:
+            if widget.handler_is_connected(self.handlers[widget]):
+                widget.handler_block(self.handlers[widget])
 
     def unblock_handlers(self):
-        for (widget, handler) in self.handlers:
-            if widget.handler_is_connected(handler):
-                widget.handler_unblock(handler)
+        for widget in self.handlers:
+            if widget.handler_is_connected(self.handlers[widget]):
+                widget.handler_unblock(self.handlers[widget])
 
     def show_distro(self):
         """Fill the distro user interface with life"""
@@ -460,6 +462,11 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         except dbus.DBusException as e:
             if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
                 logging.error("Authentication canceled, changes have not been saved")
+                
+                combo_handler = self.handlers[self.combobox_security_updates]
+                self.combobox_security_updates.handler_block(combo_handler)
+                self.set_security_update_level()
+                self.combobox_security_updates.handler_unblock(combo_handler)
 
     def set_other_update_automation_level(self, widget):
         """Set the other update automation level to the given value via gconf"""
@@ -482,6 +489,13 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         except dbus.DBusException as e:
             if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
                 logging.error("Authentication canceled, changes have not been saved")
+                
+                combo_handler = self.handlers[self.combobox_release_upgrades]
+                self.combobox_release_upgrades.handler_block(combo_handler)
+                i = self.get_release_upgrades_policy()
+                self.combobox_release_upgrades.set_active(i)
+                self.combobox_release_upgrades.handler_unblock(combo_handler)
+                
 
     def on_combobox_server_changed(self, combobox):
         """
@@ -633,10 +647,9 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         cell_toggle = Gtk.CellRendererToggle()
         cell_toggle.set_property("xpad", 2)
         cell_toggle.set_property("ypad", 2)
-        self.handlers.append([cell_toggle,
-                              cell_toggle.connect('toggled',
+        self.handlers[cell_toggle] = cell_toggle.connect('toggled',
                                                   self.on_isv_source_toggled,
-                                                  self.cdrom_store)])
+                                                  self.cdrom_store)
         col_active = Gtk.TreeViewColumn(_("Active"), cell_toggle,
                                         active=COLUMN_ACTIVE)
 
@@ -653,10 +666,9 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         cell_toggle = Gtk.CellRendererToggle()
         cell_toggle.set_property("xpad", 2)
         cell_toggle.set_property("ypad", 2)
-        self.handlers.append([cell_toggle,
-                              cell_toggle.connect('toggled',
+        self.handlers[cell_toggle] = cell_toggle.connect('toggled',
                                                   self.on_isv_source_toggled,
-                                                  self.source_store)])
+                                                  self.source_store)
         col_active = Gtk.TreeViewColumn(_("Active"), cell_toggle,
                                         active=COLUMN_ACTIVE)
 
@@ -833,6 +845,15 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             except dbus.DBusException as e:
                 if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
                     logging.error("Authentication canceled, changes have not been saved")
+                    
+                    update_days = self.get_update_interval()
+                    combo_handler = self.handlers[self.combobox_update_interval]
+                    for key in self.combobox_interval_mapping:
+        	            if self.combobox_interval_mapping[key] == update_days:
+	                        self.combobox_update_interval.handler_block(combo_handler)
+	                        self.combobox_update_interval.set_active(key)
+	                        self.combobox_update_interval.handler_unblock(combo_handler)
+	                        break
 
     def on_add_clicked(self, widget):
         """Show a dialog that allows to enter the apt line of a to be used repo"""
